﻿namespace EmailClient.Data
{
    public record struct Response<T>(T? Result, DateTimeOffset Timestamp);

    public class MailServer
    {
        private record struct Email(int Folder, string Sender, string Recipient, string Subject, string Content, DateTimeOffset Timestamp, bool IsUnread, DateTimeOffset LastModified);
        private record struct Folder(string Name, int Parent, DateTimeOffset LastModified);

        private readonly Dictionary<int, Folder> _folders;
        private readonly Dictionary<int, Email> _emails;

        private int _lastId;
        private readonly int _sentFolderId;

        public MailServer(string address)
        {
            Address = address;
            _folders = new () {
                { -1, new (string.Empty, -1, DateTimeOffset.Now)},
                { 0, new("Inbox", -1, DateTimeOffset.Now) },
                { 1, new("Sent", -1 , DateTimeOffset.Now) },
                { 2, new("Drafts", 1, DateTimeOffset.Now) },
                { 3, new("Trash", -1, DateTimeOffset.Now) },
                { 4, new( "Spam", -1, DateTimeOffset.Now) }
            };
            _emails = new()
            {
                {
                    0,
                    new(0, "upirie0@census.gov", Address, "Diverse directional success",
                        "Duis bibendum. Morbi non quam nec dui luctus rutrum. Nulla tellus.",
                        DateTimeOffset.Parse("2021-08-01T15:43:29Z"), true, DateTimeOffset.Now)
                },
                {
                    1,
                    new(0, "mbales1@shop-pro.jp", Address, "Cross-group real-time frame",
                        "Aliquam quis turpis eget elit sodales scelerisque. Mauris sit amet eros. Suspendisse accumsan tortor quis turpis.\n\nSed ante. Vivamus tortor. Duis mattis egestas metus.".ReplaceLineEndings(),
                        DateTimeOffset.Parse("2021-07-23T06:10:34Z"), false, DateTimeOffset.Now)
                },
                {
                    2,
                    new(0, "kcurr2@smugmug.com", Address, "Configurable actuating extranet",
                        "Curabitur gravida nisi at nibh. In hac habitasse platea dictumst. Aliquam augue quam, sollicitudin vitae, consectetuer eget, rutrum at, lorem.\n\nInteger tincidunt ante vel ipsum. Praesent blandit lacinia erat. Vestibulum sed magna at nunc commodo placerat.".ReplaceLineEndings(),
                        DateTimeOffset.Parse("2021-07-26T06:15:06Z"), true, DateTimeOffset.Now)
                },
                {
                    3,
                    new(1, Address, "cgrimsey3@digg.com", "Reactive global software",
                        "Curabitur at ipsum ac tellus semper interdum. Mauris ullamcorper purus sit amet nulla. Quisque arcu libero, rutrum ac, lobortis vel, dapibus at, diam.".ReplaceLineEndings(),
                        DateTimeOffset.Parse("2021-08-30T20:04:52Z"), true, DateTimeOffset.Now)
                },
                {
                    4,
                    new(1, Address, "acroasdale4@hugedomains.com",
                        "Exclusive 4th generation throughput",
                        "In hac habitasse platea dictumst. Etiam faucibus cursus urna. Ut tellus.".ReplaceLineEndings(),
                        DateTimeOffset.Parse("2021-07-30T22:14:54Z"), false, DateTimeOffset.Now)
                },
                {
                    5,
                    new(2, Address, "rfishbourne5@g.co", "Total incremental benchmark",
                        "Morbi non lectus. Aliquam sit amet diam in magna bibendum imperdiet. Nullam orci pede, venenatis non, sodales sed, tincidunt eu, felis.\n\nFusce posuere felis sed lacus. Morbi sem mauris, laoreet ut, rhoncus aliquet, pulvinar sed, nisl. Nunc rhoncus dui vel sem.\n\nSed sagittis. Nam congue, risus semper porta volutpat, quam pede lobortis ligula, sit amet eleifend pede libero quis orci. Nullam molestie nibh in lectus.".ReplaceLineEndings(),
                        DateTimeOffset.Parse("2021-07-02T21:07:15Z"), false, DateTimeOffset.Now)
                },
                {
                    6,
                    new(3, "mwedlake6@360.cn", Address, "Function-based client-driven core",
                        "Fusce posuere felis sed lacus. Morbi sem mauris, laoreet ut, rhoncus aliquet, pulvinar sed, nisl. Nunc rhoncus dui vel sem.\n\nSed sagittis. Nam congue, risus semper porta volutpat, quam pede lobortis ligula, sit amet eleifend pede libero quis orci. Nullam molestie nibh in lectus.\n\nPellentesque at nulla. Suspendisse potenti. Cras in purus eu magna vulputate luctus.".ReplaceLineEndings(),
                        DateTimeOffset.Parse("2022-01-21T03:59:08Z"), false, DateTimeOffset.Now)
                },
                {
                    7,
                    new(3, "cmurtagh7@yellowbook.com", Address, "Enterprise-wide tertiary middleware",
                        "Integer ac leo. Pellentesque ultrices mattis odio. Donec vitae nisi.\n\nNam ultrices, libero non mattis pulvinar, nulla pede ullamcorper augue, a suscipit nulla elit ac nulla. Sed vel enim sit amet nunc viverra dapibus. Nulla suscipit ligula in lacus.".ReplaceLineEndings(),
                        DateTimeOffset.Parse("2021-10-24T03:38:19Z"), false, DateTimeOffset.Now)
                },
                {
                    8,
                    new(3, "glongcaster8@cbslocal.com", Address, "Centralized cohesive success",
                        "Cras mi pede, malesuada in, imperdiet et, commodo vulputate, justo. In blandit ultrices enim. Lorem ipsum dolor sit amet, consectetuer adipiscing elit.\n\nProin interdum mauris non ligula pellentesque ultrices. Phasellus id sapien in sapien iaculis congue. Vivamus metus arcu, adipiscing molestie, hendrerit at, vulputate vitae, nisl.\n\nAenean lectus. Pellentesque eget nunc. Donec quis orci eget orci vehicula condimentum.".ReplaceLineEndings(),
                        DateTimeOffset.Parse("2021-06-16T03:33:59Z"), false, DateTimeOffset.Now)
                },
                {
                    9,
                    new(4, "iwragge9@tripadvisor.com", Address, "Secured static knowledge user",
                        "Phasellus sit amet erat. Nulla tempus. Vivamus in felis eu sapien cursus vestibulum.\n\nProin eu mi. Nulla ac enim. In tempor, turpis nec euismod scelerisque, quam turpis adipiscing lorem, vitae mattis nibh ligula nec sem.".ReplaceLineEndings(),
                        DateTimeOffset.Parse("2021-07-25T14:42:31Z"), false, DateTimeOffset.Now)
                }
            };
            _lastId = 9;
            _sentFolderId = 1;
        }
        public string Address { get; }

        public bool HasAccountChanged(DateTimeOffset lastSync) => LastAccountUpdate > lastSync;

        public DateTimeOffset LastAccountUpdate => _folders[-1].LastModified;

        public Response<IEnumerable<int>> GetFolders() =>
            new Response<IEnumerable<int>>(
                _folders.Where((f) => f.Key != -1 && f.Value.Parent == -1).Select(f => f.Key),
                _folders[-1].LastModified);

        public Response<int> GetSentFolder() => new Response<int>(_sentFolderId, _folders[-1].LastModified);

        private Response<T> GetFolderResponse<T>(int id, Func<Folder, T> f, T? def = default)
        {
            if (id < 0 || !_folders.ContainsKey(id))
                return new Response<T>(def, DateTimeOffset.MinValue);
            return new Response<T>(f(_folders[id]), _folders[id].LastModified);
        }

        public Response<string> GetFolderName(int id) => GetFolderResponse(id, f => f.Name, string.Empty);

        private Response<DateTimeOffset> SetFolderResponse(int id, Func<Folder, Folder> f)
        {
            if (id < 0 || !_folders.ContainsKey(id))
                return new Response<DateTimeOffset>(DateTimeOffset.MinValue, DateTimeOffset.MinValue);
            var lastUpdate = _folders[id].LastModified;
            var stamp = DateTimeOffset.Now;
            _folders[id] = f(_folders[id]) with {LastModified = stamp};
            return new Response<DateTimeOffset>(lastUpdate, stamp);
        }

        public Response<DateTimeOffset> SetFolderName(int id, string name) =>
            SetFolderResponse(id, f => f with {Name = name});

        public Response<IEnumerable<int>> GetSubFolders(int id) =>
            GetFolderResponse(id, f => _folders.Where(kv => kv.Value.Parent == id).Select(kv => kv.Key));

        public Response<(DateTimeOffset SrcLast, DateTimeOffset TargetLast)> MoveFolder(int id, int parentId)
        {
            if (id < 0 || !_folders.ContainsKey(id))
                return new Response<(DateTimeOffset srcLast, DateTimeOffset targetLast)>(
                    (DateTimeOffset.MinValue, DateTimeOffset.MinValue), DateTimeOffset.MinValue);
            int oldParent = _folders[id].Parent;
            var srcLast = _folders[oldParent].LastModified;
            if (!_folders.ContainsKey(parentId) || oldParent == parentId)
                return new Response<(DateTimeOffset srcLast, DateTimeOffset targetLast)>(
                    (srcLast, DateTimeOffset.MinValue), DateTimeOffset.MinValue);
            var targetLast = _folders[parentId].LastModified;
            var stamp = DateTimeOffset.Now;
            _folders[oldParent] = _folders[oldParent] with {LastModified = stamp};
            _folders[parentId] = _folders[parentId] with {LastModified = stamp};
            _folders[id] = _folders[id] with {Parent = parentId};
            return new Response<(DateTimeOffset SrcLast, DateTimeOffset TargetLast)>((srcLast, targetLast), stamp);
        }

        public bool HasFolderChanged(int id, DateTimeOffset lastSync) =>
            LastFolderUpdate(id) > lastSync;

        public DateTimeOffset LastFolderUpdate(int id) => id >= 0 && _folders.ContainsKey(id)
            ? _folders[id].LastModified
            : DateTimeOffset.MinValue;


        public Response<IEnumerable<int>> GetFolderEmails(int id) =>
            GetFolderResponse(id, _ => _emails.Where(e => e.Value.Folder == id).Select(e => e.Key));
        
        private Response<T> GetEmailResponse<T>(int id, Func<Email, T> f, T? def = default)
        {
            if (!_emails.ContainsKey(id))
                return new Response<T>(def, DateTimeOffset.MinValue);
            return new Response<T>(f(_emails[id]), _emails[id].LastModified);
        }

        private Response<DateTimeOffset> SetEmailResponse(int id, Func<Email, Email> f)
        {
            if (!_emails.ContainsKey(id))
                return new Response<DateTimeOffset>(DateTimeOffset.MinValue, DateTimeOffset.MinValue);
            var last = _emails[id].LastModified;
            var stamp = DateTimeOffset.Now;
            _emails[id] = f(_emails[id]) with { LastModified = stamp };
            return new Response<DateTimeOffset>(last, stamp);
        }

        public Response<string> GetEmailSubject(int id) => GetEmailResponse(id, e => e.Subject, string.Empty);

        public Response<DateTimeOffset> SetEmailSubject(int id, string subject) =>
            SetEmailResponse(id, (e => e with {Subject = subject}));

        public Response<string> GetEmailContent(int id) => GetEmailResponse(id, e => e.Content, string.Empty);

        public Response<DateTimeOffset> SetEmailContent(int id, string content) =>
            SetEmailResponse(id, e => e with {Content = content});

        public Response<string> GetEmailRecipient(int id) => GetEmailResponse(id, e => e.Recipient, string.Empty);

        public Response<DateTimeOffset> SetEmailRecipient(int id, string address) =>
            SetEmailResponse(id, e => e with {Recipient = address});

        public Response<string> GetEmailSender(int id) => GetEmailResponse(id, e => e.Sender, string.Empty);

        public Response<DateTimeOffset> SetEmailSender(int id, string sender) =>
            SetEmailResponse(id, e => e with {Sender = sender});

        public Response<DateTimeOffset> GetEmailTimestamp(int id) =>
            GetEmailResponse(id, e => e.Timestamp, DateTimeOffset.MinValue);

        public Response<DateTimeOffset> SetEmailTimestamp(int id, DateTimeOffset timestamp) =>
            SetEmailResponse(id, e => e with {Timestamp = timestamp});

        public Response<bool> GetEmailStatus(int id) => GetEmailResponse(id, e => e.IsUnread);

        public Response<DateTimeOffset> SetEmailStatus(int id, bool isUnread) =>
            SetEmailResponse(id, e => e with {IsUnread = isUnread});
        
        public Response<(DateTimeOffset SrcLast, DateTimeOffset TargetLast)> MoveEmail(int emailId, int folderId)
        {
            if (!_emails.ContainsKey(emailId))
                return new Response<(DateTimeOffset SrcLast, DateTimeOffset TargetLast)>(
                    (DateTimeOffset.MinValue, DateTimeOffset.MinValue), DateTimeOffset.MinValue);
            var oldFolder = _emails[emailId].Folder;
            var srcLast = _folders[oldFolder].LastModified;
            if (!_folders.ContainsKey(folderId) || oldFolder == folderId)
                return new Response<(DateTimeOffset SrcLast, DateTimeOffset TargetLast)>(
                    (srcLast, DateTimeOffset.MinValue), DateTimeOffset.MinValue);
            var targetLast = _folders[folderId].LastModified;
            var stamp = DateTimeOffset.Now;
            _emails[emailId] = _emails[emailId] with {Folder = folderId}; //intentionally no change to LastModified!
            _folders[oldFolder] = _folders[oldFolder] with {LastModified = stamp};
            _folders[folderId] = _folders[folderId] with {LastModified = stamp};
            return new Response<(DateTimeOffset SrcLast, DateTimeOffset TargetLast)>((srcLast, targetLast), stamp);
        }

        public Response<(int EmailID, DateTimeOffset FolderLast)> AddEmail(int folderId)
        {
            if (folderId < 0 || !_folders.ContainsKey(folderId))
                return new Response<(int EmailID, DateTimeOffset FolderLast)>((-1, DateTimeOffset.MinValue),
                    DateTimeOffset.MinValue);
            int id = ++_lastId;
            var folderLast = _folders[folderId].LastModified;
            var stamp = DateTimeOffset.Now;
            _emails.Add(id, new(folderId, string.Empty, string.Empty, string.Empty, string.Empty, stamp, true, stamp));
            _folders[folderId] = _folders[folderId] with {LastModified = stamp};
            return new Response<(int EmailID, DateTimeOffset FolderLast)>((id, folderLast), stamp);
        }
        
        public Response<DateTimeOffset> DeleteEmail(int id)
        {
            if (!_emails.ContainsKey(id)) return new Response<DateTimeOffset>(DateTimeOffset.MinValue, DateTimeOffset.MinValue);
            var folderId = _emails[id].Folder;
            var last = _folders[folderId].LastModified;
            var stamp = DateTimeOffset.Now;
            _folders[folderId] = _folders[folderId] with {LastModified = stamp };
            _emails.Remove(id);
            return new Response<DateTimeOffset>(last, stamp);
        }

        public Response<(int EmailID, DateTimeOffset FolderLast)> SendEmail(string subject, string content, string recipient)
        {
            int id = ++_lastId;
            var last = _folders[_sentFolderId].LastModified;
            var stamp = DateTimeOffset.Now;
            _emails.Add(id, new(_sentFolderId, Address, recipient, subject, content, stamp, false, stamp));
            _folders[_sentFolderId] = _folders[1] with {LastModified = stamp};
            return new Response<(int EmailID, DateTimeOffset FolderLast)>((id, last), stamp);
        }
        
        public bool HasEmailChanged(int id, DateTimeOffset lastSync) =>
            LastEmailUpdate(id) > lastSync;

        public DateTimeOffset LastEmailUpdate(int id) =>
            _emails.ContainsKey(id) ? _emails[id].LastModified : DateTimeOffset.MinValue;

    }
}