﻿using System.Collections.ObjectModel;
using EmailClient.DomainModel.Component;

namespace EmailClient.DomainModel
{
    public class Folder : ObservableObject
    {
        private string _name;

        internal Folder(int id, string name, DateTimeOffset lastSync, IEnumerable<Email> emails, IEnumerable<Folder> subFolders)
        {
            Id = id;
            _name = name;
            LastSync = lastSync;
            Emails = new ObservableCollection<Email>(emails);
            SubFolders = new ObservableCollection<Folder>(subFolders);
        }

        internal int Id { get; }

        internal DateTimeOffset LastSync { get; set; }

        public string Name
        {
            get => _name;
            set => SetValue(ref _name, ref value);
        }

        public ObservableCollection<Email> Emails { get; }

        public ObservableCollection<Folder> SubFolders { get; }

        internal Email? FindEmail(int emailId)
        {
            foreach (var email in Emails)
            {
                if (email.Id == emailId)
                    return email;
            }

            return null;
        }
    }
}
