﻿using EmailClient.DomainModel;

namespace EmailClient.MVC.View
{
    public interface IEmailEditController
    {
        Email Email { get; }
        void Send();
        bool CloseUnsent(Form? owner);
    }

    public partial class EmailEditView : Form
    {
        private readonly IEmailEditController _controller;
        private readonly Email _model;

        public EmailEditView(IEmailEditController controller)
        {
            InitializeComponent();
            _controller = controller;
            _model = _controller.Email;
            tbAddress.Text = _model.Recipient;
            tbSubject.Text = _model.Subject;
            tbContent.Text = _model.Content;
        }

        private void UpdateEmail()
        {
            _model.Recipient = tbAddress.Text;
            _model.Subject = tbSubject.Text;
            _model.Content = tbContent.Text;
        }

        private void tSyncChanges_Tick(object sender, EventArgs e)
        {
            UpdateEmail();
        }

        private void btnSend_Click(object sender, EventArgs e)
        {
            SendEmail();
        }

        private void SendEmail()
        {
            UpdateEmail();
            _controller.Send();
            Close();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void EmailEditView_FormClosing(object sender, FormClosingEventArgs e)
        {
            UpdateEmail();
            e.Cancel = !_controller.CloseUnsent(this);
        }

        private void EmailEditView_Shown(object sender, EventArgs e)
        {
            tbAddress.SelectionStart = tbAddress.Text.Length;
            tbSubject.SelectionStart = tbSubject.Text.Length;
            tbContent.SelectionStart = tbContent.Text.Length;
            tbAddress.SelectionLength = tbSubject.SelectionLength = tbContent.SelectionLength = 0;
            if (!string.IsNullOrEmpty(tbAddress.Text))
            {
                if (!string.IsNullOrEmpty(tbSubject.Text))
                {
                    tbContent.Focus();
                }
                else
                    tbSubject.Focus();
            }
        }

        private void tbContent_PreviewKeyDown(object sender, PreviewKeyDownEventArgs e)
        {
            if (e.KeyCode == Keys.Return && e.Modifiers == Keys.Control)
                SendEmail();
        }
    }
}
