﻿using System.ComponentModel;
using EmailClient.DomainModel;

namespace EmailClient.MVC.View
{
    public interface IEmailController
    {
        Email Email { get; }
        void ComposeReplyEmail(Form? owner);
        void ComposeForwardEmail(Form? owner);
    }

    public partial class EmailView : Form
    {
        private readonly IEmailController _controller;
        private readonly Email _model;

        private readonly string _timestampFormat = "yyyy-MM-dd HH:mm";

        public EmailView(IEmailController controller)
        {
            InitializeComponent();
            _controller = controller;
            _model = controller.Email;
            lblAddress.Text = _model.Recipient;
            lblSubject.Text = _model.Subject;
            lblTimestamp.Text = _model.Timestamp.ToString(_timestampFormat);
            tbContent.Text = _model.Content;
            _model.PropertyChanged += UpdateEmail;
        }

        private void UpdateEmail(object? sender, PropertyChangedEventArgs e)
        {
            if (sender is Email email)
                switch (e.PropertyName)
                {
                    case nameof(Email.Recipient):
                        lblAddress.Text = email.Recipient;
                        break;
                    case nameof(Email.Subject):
                        lblSubject.Text = email.Subject;
                        break;
                    case nameof(Email.Timestamp):
                        lblTimestamp.Text = email.Timestamp.ToString(_timestampFormat);
                        break;
                    case nameof(Email.Content):
                        tbContent.Text = email.Content;
                        break;
                }
        }

        private void btnReplyTo_Click(object sender, EventArgs e)
        {
            _controller.ComposeReplyEmail(Owner);
        }

        private void btnForwardEmail_Click(object sender, EventArgs e)
        {
            _controller.ComposeForwardEmail(Owner);
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void EmailView_FormClosed(object sender, FormClosedEventArgs e)
        {
            _model.PropertyChanged -= UpdateEmail;
        }
    }
}
