﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using EmailClient.MVP.View;
using EmailClient.Transactional.Entity;
using EmailClient.Transactional.Service;

namespace EmailClient.MVP.Presenter
{
    public class EmailEditPresenter
    {
        private readonly Email _email;
        private readonly int _parentFolderId;
        private readonly IEmailEditView _view;
        private bool _sent = false;

        public event EmailEventHandler? UpdateExecuted;
        public event EmailEventHandler? SentExecuted;
        public event EmailEventHandler? DeleteExecuted;

        public EmailEditPresenter(Email email, int parentFolderId, IEmailEditView view)
        {
            _email = email;
            _parentFolderId = parentFolderId;
            _view = view;
            _view.UpdateExecuted += SaveEmail;
            _view.SendExecuted += SendEmail;
            _view.CancelExecuting += ConfirmClose;
            _view.CancelExecuted += RemoveHandlers;
            _view.EmailContent = email.Content;
            _view.EmailRecipient = email.Recipient;
            _view.EmailSubject = email.Subject;
        }

        private void SaveEmail(object? sender, EventArgs e)
        {
            OnUpdateExecuted(new (_parentFolderId, _email));
        }

        private void RemoveHandlers(object? sender, EventArgs e)
        {
            _view.UpdateExecuted -= SaveEmail;
            _view.SendExecuted -= SendEmail;
            _view.CancelExecuting -= ConfirmClose;
            _view.CancelExecuted -= RemoveHandlers;
        }

        private void ConfirmClose(object? sender, CancelEventArgs e)
        {
            if (_sent) return;
            var res = _view.ShowCloseConfirmationDialog();
            switch (res)
            {
                case DialogResult.Cancel:
                    e.Cancel = true;
                    break;
                case DialogResult.No:
                    OnDeleteExecuted(new (_parentFolderId, _email));
                    break;
            }
        }

        private void SendEmail(object? sender, EventArgs e)
        {
            OnSentExecuted(new (_parentFolderId, _email));
            OnDeleteExecuted(new (_parentFolderId, _email));
            _sent = true;
            _view.Close();
        }

        protected virtual void OnSentExecuted(EmailEventArgs e)
        {
            SentExecuted?.Invoke(this, e);
        }

        protected virtual void OnDeleteExecuted(EmailEventArgs e)
        {
            DeleteExecuted?.Invoke(this, e);
        }

        protected virtual void OnUpdateExecuted(EmailEventArgs e)
        {
            UpdateExecuted?.Invoke(this, e);
        }
    }
}
