﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using EmailClient.MVP.View;
using EmailClient.Transactional.Entity;
using EmailClient.Transactional.Service;

namespace EmailClient.MVP.Presenter
{
    public class EmailPresenter
    {
        private readonly IAccountService _modelService;
        private readonly IEmailView _view;
        private Email _email;
        private readonly int _parentFolderId;

        public event EmailEventHandler? ReplyToExecuted;
        public event EmailEventHandler? ForwardEmailExecuted;

        public EmailPresenter(IAccountService modelService, Email email, int parentFolderId, IEmailView view)
        {
            _email = email;
            _parentFolderId = parentFolderId;
            _modelService = modelService;
            _view = view;
            _modelService.EmailChanged += UpdateEmailInView;
            _view.ViewClosed += RemoveHandlers;
            _view.ForwardToExecuted += ForwardEmail;
            _view.ReplyToExecuted += ReplyToEmail;
            _view.SetEmailSubject(email.Subject);
            _view.SetEmailAddress(email.Sender == modelService.GetAccountAddress() ? email.Recipient : email.Sender);
            _view.SetEmailContent(email.Content);
            _view.SetEmailDate(email.Timestamp.ToString("yyyy-MM-dd HH:mm"));
        }

        private void UpdateEmailInView(object sender, EmailEventArgs e)
        {
            if (e.Email.Id != _email.Id) return;
            _email = e.Email;
            UpdateEmailInViewImpl();
        }

        private void UpdateEmailInViewImpl()
        {
            _view.SetEmailAddress(_email.Recipient == _modelService.GetAccountAddress()
                ? _email.Sender
                : _email.Recipient);
            _view.SetEmailSubject(_email.Subject);
            _view.SetEmailContent(_email.Content);
        }

        private void ReplyToEmail(object? sender, EventArgs e)
        {
            OnReplyToExecuted(new (_parentFolderId, _email));
        }

        private void ForwardEmail(object? sender, EventArgs e)
        {
            OnForwardEmailExecuted(new (_parentFolderId, _email));
        }

        private void RemoveHandlers(object? sender, EventArgs e)
        {
            _modelService.EmailChanged -= UpdateEmailInView;
            _view.ForwardToExecuted -= ForwardEmail;
            _view.ReplyToExecuted -= ReplyToEmail;
            _view.ViewClosed -= RemoveHandlers;
        }

        protected virtual void OnReplyToExecuted(EmailEventArgs e)
        {
            ReplyToExecuted?.Invoke(this, e);
        }

        protected virtual void OnForwardEmailExecuted(EmailEventArgs e)
        {
            ForwardEmailExecuted?.Invoke(this, e);
        }
    }
}
