﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using EmailClient.Transactional.Service;

namespace EmailClient.MVP.View
{
    public interface IEmailEditView
    {
        event EventHandler? UpdateExecuted;
        event EventHandler? SendExecuted;
        event EventHandler<CancelEventArgs>? CancelExecuting;
        event EventHandler? CancelExecuted;
        DialogResult ShowCloseConfirmationDialog();
        void Close();
        string EmailRecipient { get; set; }
        string EmailSubject { get; set; }
        string EmailContent { get; set; }
    }

    public partial class EmailEditView : Form, IEmailEditView
    {
        public event EventHandler? UpdateExecuted;
        public event EventHandler? SendExecuted;
        public event EventHandler<CancelEventArgs>? CancelExecuting;
        public event EventHandler? CancelExecuted;

        public EmailEditView(Form? owner)
        {
            InitializeComponent();
            Owner = owner;
        }

        public string EmailRecipient
        {
            get => tbAddress.Text;
            set => tbAddress.Text = value;
        }

        public string EmailSubject
        {
            get => tbSubject.Text;
            set => tbSubject.Text = value;
        }

        public string EmailContent
        {
            get => tbContent.Text;
            set => tbContent.Text = value;
        }

        public DialogResult ShowCloseConfirmationDialog()
        {
            return MessageBox.Show(this, "Do you want to preserve the email draft?",
                "Unsent Email Draft", MessageBoxButtons.YesNoCancel);
        }

        protected virtual void OnSendExecuted()
        {
            SendExecuted?.Invoke(this, EventArgs.Empty);
        }

        protected virtual void OnCancelExecuting(CancelEventArgs e)
        {
            CancelExecuting?.Invoke(this, e);
        }

        protected virtual void OnCancelExecuted()
        {
            CancelExecuted?.Invoke(this, EventArgs.Empty);
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void btnSend_Click(object sender, EventArgs e)
        {
            OnSendExecuted();
            Close();
        }

        private void EmailEditView_FormClosing(object sender, FormClosingEventArgs e)
        {
            OnCancelExecuting(e);
        }

        private void EmailEditView_FormClosed(object sender, FormClosedEventArgs e)
        {
            OnCancelExecuted();
        }

        private void tSyncChanges_Tick(object sender, EventArgs e)
        {
            OnUpdateExecuted();
        }

        protected virtual void OnUpdateExecuted()
        {
            UpdateExecuted?.Invoke(this, EventArgs.Empty);
        }
    }
}
