﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace EmailClient.MVP.View
{
    public interface IEmailView
    {
        event EventHandler? ViewClosed;
        event EventHandler? ReplyToExecuted;
        event EventHandler? ForwardToExecuted;
        void SetEmailAddress(string address);
        void SetEmailSubject(string subject);
        void SetEmailContent(string content);
        void SetEmailDate(string date);
    }

    public partial class EmailView : Form, IEmailView
    {
        public event EventHandler? ViewClosed;
        public event EventHandler? ReplyToExecuted;
        public event EventHandler? ForwardToExecuted;
        public EmailView(Form? owner = null)
        {
            InitializeComponent();
            Owner = owner;
        }

        public void SetEmailAddress(string address)
        {
            lblAddress.Text = address;
        }

        public void SetEmailSubject(string subject)
        {
            lblSubject.Text = subject;
        }

        public void SetEmailContent(string content)
        {
            tbContent.Text = content;
        }

        public void SetEmailDate(string date)
        {
            lblTimestamp.Text = date;
        }

        protected virtual void OnViewClosed()
        {
            ViewClosed?.Invoke(this, EventArgs.Empty);
        }

        private void EmailView_FormClosed(object sender, FormClosedEventArgs e)
        {
            OnViewClosed();
        }

        protected virtual void OnReplyToExecuted()
        {
            ReplyToExecuted?.Invoke(this, EventArgs.Empty);
        }

        protected virtual void OnForwardToExecuted()
        {
            ForwardToExecuted?.Invoke(this, EventArgs.Empty);
        }

        private void btnReplyTo_Click(object sender, EventArgs e)
        {
            OnReplyToExecuted();
        }

        private void btnForwardEmail_Click(object sender, EventArgs e)
        {
            OnForwardToExecuted();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
