﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Media3D;

namespace EmailClient.MVVM.Behaviors
{
    public static class SelectorBehaviors
    {
        public static readonly DependencyProperty EmptyAreaDeselectProperty =
            DependencyProperty.RegisterAttached("EmptyAreaDeselect", typeof(bool), typeof(SelectorBehaviors),
                new(false, EmptyAreaDeselectPropertyChanged));

        public static void SetEmptyAreaDeselect(UIElement element, bool value)
        {
            if (GetEmptyAreaDeselect(element) != value)
                element.SetValue(EmptyAreaDeselectProperty, value);
        }

        public static bool GetEmptyAreaDeselect(UIElement element)
        {
            return (bool)element.GetValue(EmptyAreaDeselectProperty);
        }

        private static void SelectorMouseDown(object sender, MouseButtonEventArgs e)
        {
            if (sender is Selector selector)
                selector.SelectedIndex = -1;
            else if (sender is TreeView tree)
            {
                if (tree.SelectedItem != null)
                {
                    var container = tree.ItemContainerGenerator.ContainerFromItem(tree.SelectedItem);
                    if (container != null)
                        ((TreeViewItem)container).IsSelected = false;
                }
            }
        }

        private static void EmptyAreaDeselectPropertyChanged(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            if (dependencyObject is not Selector && dependencyObject is not TreeView)
                throw new ArgumentException("Expected Selector subclass or a TreeView", nameof(dependencyObject));
            UIElement element = (UIElement)dependencyObject;
            if (e.OldValue == e.NewValue)
                return;
            if (!(bool)e.NewValue)
                element.MouseDown -= SelectorMouseDown;
            else
                element.MouseDown += SelectorMouseDown;
        }
    }
}
