﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using EmailClient.DomainModel;
using EmailClient.MVVM.Services;

namespace EmailClient.MVVM.ViewModel
{
    public partial class AccountViewModel : ObservableObject
    {
        private Account _account;

        [ObservableProperty]
        [NotifyCanExecuteChangedFor(nameof(ReplyCommand))]
        [NotifyCanExecuteChangedFor(nameof(ForwardCommand))]
        private EmailViewModel? _selectedEmail;

        public AccountViewModel(Account account, string accountAddress)
        {
            _account = account;
            Folders = new ObservableCollection<FolderViewModel>(_account.Folders.Select(f =>
                new FolderViewModel(_account, f)));
            var f = Folders.FirstOrDefault();
            if (f != null)
                f.IsSelected = true;
        }

        public ObservableCollection<FolderViewModel> Folders { get; }

        [RelayCommand]
        void RefreshFolder(FolderViewModel? folder)
        {
            if (folder == null)
            {
                _account.RefreshAll();
            }
            else
            {
                _account.RefreshFolder(folder.Model);
            }
        }

        [RelayCommand(CanExecute = nameof(IsEmailSelected))]
        void Reply()
        {
            if (SelectedEmail != null)
                SelectedEmail.Reply();
        }

        [RelayCommand(CanExecute = nameof(IsEmailSelected))]
        void Forward()
        {
            if (SelectedEmail != null)
                SelectedEmail.Forward();
        }

        [RelayCommand]
        void Compose()
        {
            var email = _account.CreateDraft();
            if (email != null)
                DialogServiceLocator.Instance.EditDetails(new EmailViewModel(_account, _account.DraftFolder!, email));
        }

        [RelayCommand]
        void ShowDetails(EmailViewModel? model)
        {
            if (model != null)
                model.ShowDetails();
        }

        bool IsEmailSelected()
        {
            return SelectedEmail != null;
        }
        
        internal Account Model => _account;
    }
}
