﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using EmailClient.Data;

namespace EmailClient.Monolith
{
    public partial class EditEmailWindow : Form
    {
        private readonly MailServer _server;
        private int _draftId;

        public EditEmailWindow(MailServer server, int draftId)
        {
            InitializeComponent();
            _server = server;
            _draftId = draftId;
            tbAddress.Text = _server.GetEmailRecipient(_draftId).Result;
            tbSubject.Text = _server.GetEmailSubject(_draftId).Result;
            tbContent.Text = _server.GetEmailContent(_draftId).Result;
        }

        private void btnSend_Click(object sender, EventArgs e)
        {
            SendEmail();
        }

        private void SendEmail()
        {
            _server.SendEmail(tbSubject.Text, tbContent.Text, tbAddress.Text);
            _server.DeleteEmail(_draftId);
            _draftId = -1;
            Close();
        }

        private void tUpdateDraft_Tick(object sender, EventArgs e)
        {
            UpdateDraft();
        }

        private void UpdateDraft()
        {
            _server.SetEmailRecipient(_draftId, tbAddress.Text);
            _server.SetEmailSubject(_draftId, tbSubject.Text);
            _server.SetEmailContent(_draftId, tbContent.Text);
        }

        private void EditEmailWindow_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_draftId < 0) return;
            var answer = MessageBox.Show("Do you want to preserve the email draft?",
                "Unsent Email Draft", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
            if (answer == DialogResult.Cancel)
            {
                e.Cancel = true;
                return;
            }
            if (answer == DialogResult.Yes)
                UpdateDraft();
            else
                _server.DeleteEmail(_draftId);
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void tbContent_PreviewKeyDown(object sender, PreviewKeyDownEventArgs e)
        {
            if (e.KeyCode == Keys.Enter && e.Control)
                SendEmail();

        }

        private void EditEmailWindow_Shown(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(tbAddress.Text))
            {
                if (!string.IsNullOrEmpty(tbSubject.Text))
                {
                    tbContent.Focus();
                    tbContent.SelectionStart = tbContent.Text.Length;
                    tbContent.SelectionLength = 0;
                }
                else
                    tbSubject.Focus();
            }
        }
    }
}
