using EmailClient.Data;

namespace EmailClient.Monolith
{
    public partial class MainWindow : Form
    {
        private MailServer _server;

        public MainWindow()
        {
            InitializeComponent();
            _server = new MailServer("pawel.aszklar@pw.edu.pl");
        }

        private void MainWindow_Load(object sender, EventArgs e)
        {
            foreach (var folderId in _server.GetFolders().Result!)
                tvAccountFolders.Nodes.Add(CreateFolderNode(folderId));
        }

        private TreeNode CreateFolderNode(int folderId)
        {
            string name = _server.GetFolderName(folderId).Result!;
            var node = new TreeNode(name) {
                Tag = folderId
            };
            foreach (var subFolder in _server.GetSubFolders(folderId).Result!)
                node.Nodes.Add(CreateFolderNode(subFolder));
            return node;
        }

        private void tvAccountFolders_AfterSelect(object sender, TreeViewEventArgs e)
        {
            int folderId = -1;
            if (e.Node is { Tag: int })
                folderId = (int)e.Node.Tag;
            LoadFolderEmails(folderId);
        }

        private int SelectedEmailId =>
            lvFolderEmails.SelectedItems.Count > 0 && 
            lvFolderEmails.SelectedItems[0] is {Tag: int emailId}
                ? emailId
                : -1;

        private int SelectedFolderId => tvAccountFolders.SelectedNode is {Tag: int folderId} ? folderId : -1;

        private void LoadFolderEmails(int folderId)
        {
            int selectedEmail = SelectedEmailId;
            lvFolderEmails.SelectedItems.Clear(); //forces SelectedIndexChanged to fire
            lvFolderEmails.Items.Clear();
            if (folderId >= 0)
            {
                foreach (var emailId in _server.GetFolderEmails(folderId).Result!)
                {
                    var isUnread = _server.GetEmailStatus(emailId).Result;
                    var correspondent = _server.GetEmailRecipient(emailId).Result;
                    if (correspondent == _server.Address) correspondent = _server.GetEmailSender(emailId).Result;
                    var item = new ListViewItem(new[]
                    {
                        isUnread ? "*" : string.Empty,
                        _server.GetEmailSubject(emailId).Result,
                        correspondent,
                        _server.GetEmailTimestamp(emailId).Result.ToString("yyyy-MM-dd HH:mm")
                    }) {
                        Tag = emailId
                    };
                    if (isUnread)
                        item.Font = new Font(lvFolderEmails.Font, FontStyle.Bold);
                    lvFolderEmails.Items.Add(item);
                    if (emailId == selectedEmail) 
                        item.Selected = true;
                }
                btnRefreshFolder.Enabled = true;
            }
            else
                btnRefreshFolder.Enabled = false;
        }

        private void lvFolderEmails_SelectedIndexChanged(object sender, EventArgs e)
        {
            tbEmailContent.Clear();
            if (lvFolderEmails.SelectedItems.Count > 0)
            {
                var emailItem = lvFolderEmails.SelectedItems[0];
                if (emailItem.Tag is int emailId)
                {
                    tbEmailContent.Text = _server.GetEmailContent(emailId).Result;
                    var mailReadColumn = emailItem.SubItems[0];
                    if (mailReadColumn.Text.Length > 0)
                    {
                        _server.SetEmailStatus(emailId, false);
                        mailReadColumn.Text = string.Empty;
                        emailItem.Font = null;
                    }
                }
                btnReplyTo.Enabled = btnForwardEmail.Enabled = true;
            }
            else
                btnReplyTo.Enabled = btnForwardEmail.Enabled = false;
        }

        private void RefreshFolder_Fired(object sender, EventArgs e)
        {
            LoadFolderEmails(SelectedFolderId);
        }

        private int FindDraftFolder(TreeNodeCollection nodes)
        {
            foreach (TreeNode node in nodes)
            {
                if (node.Text == "Drafts")
                    return (int)node.Tag;
                int id = FindDraftFolder(node.Nodes);
                if (id >= 0) return id;
            }
            return -1;
        }

        private void btnNewEmail_Click(object sender, EventArgs e)
        {
            int draftFolder = FindDraftFolder(tvAccountFolders.Nodes);
            if (draftFolder < 0)
                return;
            int draftId = _server.AddEmail(draftFolder).Result.EmailID;
            ShowDraftEdit(draftId);
        }

        private void ShowDraftEdit(int draftId)
        {
            var emailEdit = new EditEmailWindow(_server, draftId);
            emailEdit.Owner = this;
            emailEdit.Show();
        }

        private void ShowEmailView(int emailId)
        {
            var emailView = new ViewEmailForm(_server, emailId);
            emailView.Owner = this;
            emailView.Show();
        }

        private void lvFolderEmails_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            var info = lvFolderEmails.HitTest(e.Location);
            if (info.Item is not { Tag: int emailId })
                return;
            var folderId = SelectedFolderId;
            if (folderId >= 0 && folderId == FindDraftFolder(tvAccountFolders.Nodes))
                ShowDraftEdit(emailId);
            else
                ShowEmailView(emailId);
        }

        internal void ShowReplyEdit(int emailId)
        {
            if (emailId < 0)
                return;
            var drafts = FindDraftFolder(tvAccountFolders.Nodes);
            if (drafts < 0)
                return;
            int draftId = _server.AddEmail(drafts).Result.EmailID;
            var address = _server.GetEmailSender(emailId).Result!;
            _server.SetEmailRecipient(draftId, address);
            _server.SetEmailSender(draftId, _server.Address);
            _server.SetEmailContent(draftId,
                $"{address} at {_server.GetEmailTimestamp(emailId).Result.ToString("yyyy-MM-dd HH:mm")} sent:\n{_server.GetEmailContent(emailId).Result}"
                    .ReplaceLineEndings($"{Environment.NewLine}> ") + Environment.NewLine);
            _server.SetEmailSubject(draftId, $"Re: {_server.GetEmailSubject(emailId).Result}");
            var emailEdit = new EditEmailWindow(_server, draftId);
            emailEdit.Owner = this;
            emailEdit.Show();
        }

        internal void ShowForwardEdit(int emailId)
        {
            if (emailId < 0)
                return;
            var drafts = FindDraftFolder(tvAccountFolders.Nodes);
            if (drafts < 0)
                return;
            int draftId = _server.AddEmail(drafts).Result.EmailID;
            _server.SetEmailSender(draftId, _server.Address);
            _server.SetEmailContent(draftId,
                $"----\n{_server.GetEmailSender(emailId).Result} at {_server.GetEmailTimestamp(emailId).Result.ToString("yyyy-MM-dd HH:mm")} sent:\n\n{_server.GetEmailContent(emailId).Result}\n----\n"
                    .ReplaceLineEndings());
            _server.SetEmailSubject(draftId, $"Fwd: {_server.GetEmailSubject(emailId).Result}");
            var emailEdit = new EditEmailWindow(_server, draftId);
            emailEdit.Owner = this;
            emailEdit.Show();
        }

        private void btnReplyTo_Click(object sender, EventArgs e)
        {
            ShowReplyEdit(SelectedEmailId);
        }

        private void btnForwardEmail_Click(object sender, EventArgs e)
        {
            ShowForwardEdit(SelectedEmailId);
        }
    }
}